//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Wiper.h"
#include "Global.h"
#include "Fmxutils.h"
#include <fcntl.h>
#include <io.h>
#include <dos.h>
#include <errno.h>

//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TWipeForm *WipeForm;
/*
value types:
  0: all 0's
  1: all 1's
  2: even bits = 1
  3: odd bits = 1
  4: pseudorandom
*/
int values[10][10] = {
   {  4 },
   {  0,  4 },
   {  0,  1,  4 },
   {  0,  1,  0,  4 },
   {  0,  1,  0,  1,  4 },
   {  0,  1,  0,  1,  0,  4 },
   {  0,  1,  0,  1,  0,  1,  4 },    // norton = 7 passes
   {  4,  0,  1,  0,  1,  3,  2,  4 },
   {  4,  0,  1,  0,  1,  3,  2,  3,  4},
   {  4,  0,  1,  0,  1,  3,  2,  3,  2,  4}
};

//---------------------------------------------------------------------------
int __fastcall simplewipe(AnsiString &Filename, int Passes)
{
   FILE *f;
   //int drive = Filename.LowerCase()[1]-96;
   int pc,bufsize = 32768; //clustersize(drive);
   char *outbuf = new char[bufsize+1];
   if (outbuf==NULL)
   {
      ShowMessage("Out of memory!");
      return 0;
   }
   long c,len = GetFileSize(Filename);
   String tmpname;

   if((f = fopen(Filename.c_str(), "wb")) == NULL)
   {
      ShowMessage("Unable to open "+Filename+" for wiping!");
	    return 0;
 	 }

   initrandom();
   if (Passes>10 || Passes<1) Passes = 10;
   for(pc=0; pc<Passes; pc++)
 	 {
     int type = values[Passes-1][pc];
     if (type<4)
 	      fillbuf(type,outbuf,bufsize);
	   for (c=0; c<=len; c+=bufsize)
	   {
	      if (type==4)
            fillbuf(type,outbuf,bufsize);
         if (fwrite(outbuf,bufsize,1,f) == -1)
	      {
            ShowMessage("Error writing to "+Filename+"!");
            break;
         }
	   }
	   fflush(f);
	   fseek(f,0,SEEK_SET);
 	 }
   fclose(f);

   tmpname = wipename(Filename);
   if (rename(Filename.c_str(),tmpname.c_str())>0) ;
/* {
      ShowMessage("Unable to rename "+Filename+" to "+tmpname+" for wipe!"+
         "\nError: "+LastErrorStr()
      );
   }
   else
      remove(tmpname.c_str());
*/
   if (FileExists(Filename)) DeleteFile(Filename.c_str());
   if (FileExists(tmpname)) DeleteFile(tmpname.c_str());
   delete outbuf;

   return 1;
}
//---------------------------------------------------------------------------
void __fastcall wipefiles(AnsiString &Filename, int Passes)
{
   WipeForm = new TWipeForm(Application->MainForm);
   WipeForm->files->Text = Filename;
   int i=0;
   while (i<WipeForm->files->Count)
   {
      if (!FileExists(WipeForm->files->Strings[i]))
         WipeForm->files->Delete(i);
      else
         i++;
   }
   if (WipeForm->files->Count)
   {
      WipeForm->passes = Passes;
      WipeForm->ShowModal();
   }
   delete WipeForm;
}
//---------------------------------------------------------------------------
__fastcall TWipeForm::TWipeForm(TComponent* Owner)
	: TForm(Owner)
{
   Caption = "Wipe Files";
   files = new TStringList();
   passes = 7;
}
//---------------------------------------------------------------------------
void __fastcall TWipeForm::FormDestroy(TObject *Sender)
{
   delete files;
}
//---------------------------------------------------------------------------
void __fastcall TWipeForm::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TWipeForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;

   if (files->Count)
   {
      if (FileExists(files->Strings[0]))
      {
         Label1->Caption = ExtractFileName(files->Strings[0]);
         simplewipe(files->Strings[0],passes);
      }
      files->Delete(0);
   }
   else
   {
      ModalResult = mrOk;
      return;
   }
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
int __fastcall clustersize(int drive)
{
   // this function does not return correct for disks > 2gig
   DWORD SectPerClust,BytePerSect,FreeClust,TotalClust;
   if (!GetDiskFreeSpace(NULL,&SectPerClust,&BytePerSect,&FreeClust,&TotalClust))
      ShowMessage(GetLastErrorStr());
   return ((int)BytePerSect*SectPerClust);
}
/*int __fastcall clustersize(int drive)
{
    struct dfree info;
    getdfree((unsigned char)drive,&info);
    return ((int)info.df_bsec * info.df_sclus);
}*/
//---------------------------------------------------------------------------
void __fastcall initrandom()
{
  struct time t;
  gettime(&t);
  srand(t.ti_sec^(t.ti_hund<<11));
}
void __fastcall fillbuf(int type, char *buf, int size)
{
//   int c;
   int val;
   switch(type)
   {
      case  0: val=0x00; break;//for(c=0; c<size; c++) buf[c] = 0; break;
      case  1: val=0xFF; break;//for(c=0; c<size; c++) buf[c] = -1; break;
      case  2: val=0x55; break;//for(c=0; c<size; c++) buf[c] = 0x55; break;
      case  3: val=0xAA; break;//for(c=0; c<size; c++) buf[c] = 0xaa; break;
      case  4: val=random(256);//for(c=0; c<size; c++) buf[c] = (unsigned char)random(256); break;
   }
   memset(buf,val,size);
}
AnsiString __fastcall wipename(AnsiString &oldname)
{
   String name,path = ExtractFilePath(oldname);
   do {
      name += path+FormatDateTime("hhnnss",Now());
      while (name.Length()<oldname.Length()) name += "X";
      name += ".tmp";
   }
   while (FileExists(name));
   return name;
}
//---------------------------------------------------------------------------
//
// WIPE FILE -- lifted near verbatim from
//              PGP fileio.c
//              changes are noted
//
// This is the CRC-32 polynomial, which should be okay for random
// number generation.
// x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x+1
// = 1 0000 0100 1100 0001 0001 1101 1011 0111
// = 0x04c11db7
#define POLY 0x04c11db7
static unsigned textbuf2[2 * 4096 / sizeof(unsigned)];
static void wipeout(FILE *f)
{
    unsigned *p1, *p2, *p3;
    unsigned long len;
    unsigned long t;
    int i;

    /* Get the file size */
    fseek(f, 0L, SEEK_END);
    len = ftell(f);
    rewind(f);

    /* Seed of first RNG.  Inverted to get more 1 bits */
    t = ~len;

    /* Initialize first 55 words of buf with pseudo-random stuff */
    p1 = (unsigned *) textbuf2 + 55;
    do {
 	    for (i = 0; i < 32; i++)
	       t = (t & 0x80000000) ? t << (1 ^ POLY) : t << 1;
       *--p1 = (unsigned) t;
    } while (p1 > (unsigned *) textbuf2);

    while (len) {
  	     /* Fill buffer with pseudo-random integers */
	     p3 = (unsigned *) textbuf2 + 55;
	     p2 = (unsigned *) textbuf2 + 24;
	     p1 = (unsigned *) textbuf2 + sizeof(textbuf2) / sizeof(*p1);
 	     do {
           *--p1 = *--p2 + *--p3;
 	     } while (p2 > (unsigned *) textbuf2);

	     p2 = (unsigned *) textbuf2 + sizeof(textbuf2) / sizeof(*p1);
	     do {
	         *--p1 = *--p2 + *--p3;
	     } while (p3 > (unsigned *) textbuf2);

	     p3 = (unsigned *) textbuf2 + sizeof(textbuf2) / sizeof(*p3);
	     do {
	         *--p1 = *--p2 + *--p3;
	     } while (p1 > (unsigned *) textbuf2);

	     /* Write it out - yes, we're ignoring errors */
	     if (len > sizeof(textbuf2)) {
	         fwrite((char const*)textbuf2, sizeof(textbuf2), 1, f);
	         len -= sizeof(textbuf2);
	     } else {
	         fwrite((char const*)textbuf2, len, 1, f);
	         len = 0;
	     }
    }
}

int __fastcall wipefile(AnsiString &Filename)
{
    TStringList *files = new TStringList();
    files->Text = Filename;
    for (int i=0; i<files->Count; i++)
    {
       if (!FileExists(files->Strings[i])) continue;

       FILE *f;
       if ((f = fopen(files->Strings[i].c_str(),"r+b")) == NULL)
       {
          ShowMessage("Unable to open \n"+files->Strings[i]+"\nfor wipe/delete!");
		   continue;
       }
       wipeout(f);
       fclose(f);

       remove(files->Strings[i].c_str());
    }
    return 0;
}
/*
UINT InternalFileWipe(PGPContextRef context,HWND hwnd,char *FileName)
{
	DWORD dwSize;
	DWORD dwPageSize;
	DWORD dwNumPages;
	SYSTEM_INFO sysinfo;
	char NewFileName[200+_MAX_DIR];
	char szDrive[_MAX_DRIVE];
	char szDir[ _MAX_DIR];
	char szNewFile[200-_MAX_DRIVE];
	HANDLE hFile;
	HANDLE hMap;
	LPSTR lpFile;
	int count = 0;
  BOOL CancelOperation;
	BOOL bMoveFileSuccess;
  struct _stat buf;
	UINT wipe_err = 0;
	MYSTATE *ms;
	int staterror;
	char StrRes[500];

	PGPError		error			 = kPGPError_NoErr;
	PGPDiskWipeRef	wipeRef	 = kPGPInvalidRef;
	PGPInt32		buffer[256];
	PGPInt32		passes		 = kPGPNumPatterns;
	PGPPrefRef		prefRef	 = kPGPInvalidRef;
	PGPMemoryMgrRef memMgr	 = kPGPInvalidRef;

	if( IsntPGPError(error) )
	{
		memMgr = PGPGetContextMemoryMgr(context);
		error = PGPclOpenClientPrefs (memMgr, &prefRef);
		if(IsntPGPError(error))
		{
			PGPGetPrefNumber (prefRef, kPGPPrefFileWipePasses,(PGPUInt32*)&passes);
			PGPclCloseClientPrefs (prefRef, FALSE);
		}
	}

	GetSystemInfo(&sysinfo);
	dwPageSize = sysinfo.dwPageSize;
	staterror=_stat(FileName, &buf );
	if (staterror!=0) return WIPE_NOTOPEN; // check to make sure still around

	_splitpath(FileName, szDrive, szDir, NULL, NULL);

	memset(szNewFile, 'a', sizeof(szNewFile)-1);
	szNewFile[sizeof(szNewFile)-1] = 0;
	strcpy(NewFileName, szDrive);
	strcat(NewFileName, szDir); // do in same directory
	strcat(NewFileName, szNewFile);

	bMoveFileSuccess = MoveFile(FileName, NewFileName);
	if (!bMoveFileSuccess)
	{
		DWORD dwLastError;

		dwLastError = GetLastError();

		switch (dwLastError)
		{
			case ERROR_SHARING_VIOLATION: return WIPE_ALREADYOPEN;
			case ERROR_REQUEST_ABORTED: return WIPE_USERABORT;
			case ERROR_WRITE_PROTECT: return WIPE_WRITEPROTECT;
			case ERROR_ACCESS_DENIED:
			default: return WIPE_NOTOPEN;
		}
	}

	hFile = CreateFile(	NewFileName,
						GENERIC_READ | GENERIC_WRITE,
						0,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL|FILE_FLAG_WRITE_THROUGH,
						NULL );

	if(hFile==INVALID_HANDLE_VALUE)  //  If unable to open, skip it
	{
		dwSize = GetLastError();       //  These calls are for debugging
		MoveFile(NewFileName,FileName);
		return WIPE_NOTOPEN;
	}

	dwSize = GetFileSize(hFile,NULL);        //  Get the file's size
	if( dwSize==0 )
	{
		CloseHandle(hFile);
		DeleteFile( NewFileName );		//  If nothing to erase, just kill it
		return WIPE_OK;
	}

	// we want to wipe out to next page boundary
	// Get up to one more page to make fit
	dwNumPages = (dwSize+dwPageSize-1)/dwPageSize;
	dwSize = dwNumPages * dwPageSize;

	//  Create a file mapping object
	hMap = CreateFileMapping(hFile,NULL,PAGE_READWRITE,0,0,NULL);
	if(!hMap)
	{
		dwSize = GetLastError();        //  If it didn't work, next file
		CloseHandle(hFile);
		MoveFile(NewFileName,FileName);
		return WIPE_NOFILEMAP;
	}

	//  Map the file into a memory space...
	lpFile = MapViewOfFile(hMap,FILE_MAP_WRITE,0,0,0);

	if( lpFile == NULL )
	{
		dwSize = GetLastError();              //  On error, back way out
		CloseHandle( hFile );
		CloseHandle( hMap );
		MoveFile(NewFileName,FileName);
		return WIPE_NOMEMMAP;
	}

	// we are now using Colin's patterns

	error = PGPCreateDiskWiper(context, &wipeRef, passes);
	if( IsntPGPError(error) )
	{
		while( IsntPGPError( PGPGetDiskWipeBuffer(wipeRef, buffer) ) )
		{
			DWORD count;

			for( count = 0; count < dwSize; count += 1024)
			{
				memcpy(lpFile + count, buffer, 1024);
			}

			FlushViewOfFile( lpFile, dwSize );  //  Flush it to disk
			FlushFileBuffers( hFile );
		}
		error = PGPDestroyDiskWiper(wipeRef);
	}

	UnmapViewOfFile( lpFile );	  		   //  Kill mapped area

	CloseHandle( hMap );                // Close all the handles and...
	FlushFileBuffers( hFile );
	CloseHandle( hFile );
	DeleteFile( NewFileName );          //  ...(finally) delete the file

	return WIPE_OK;
}
*/
